#!/bin/bash

# Detect Debian users running the script with "sh" instead of bash
if readlink /proc/$$/exe | grep -q "dash"; then
	echo 'This installer needs to be run with "bash", not "sh".'
	exit
fi

# Discard stdin. Needed when running from an one-liner which includes a newline
read -N 999999 -t 0.001

# Detect environments where $PATH does not include the sbin directories
if ! grep -q sbin <<< "$PATH"; then
	echo '$PATH does not include sbin. Try using "su -" instead of "su".'
	exit
fi

if [[ "$EUID" -ne 0 ]]; then
	echo "This script needs to be run with superuser privileges."
	exit
fi

new_client_setup () {
	# Given a list of the assigned internal IPv4 addresses, obtain the lowest still
	# available octet. Important to start looking at 2, because 1 is our gateway.
	octet=2
	while grep AllowedIPs /etc/wireguard/wg0.conf | cut -d "." -f 4 | cut -d "/" -f 1 | grep -q "$octet"; do
		(( octet++ ))
	done
	# Don't break the WireGuard configuration in case the address space is full
	if [[ "$octet" -eq 255 ]]; then
		echo "253 clients are already configured. The WireGuard internal subnet is full!"
		exit
	fi
	key=$(wg genkey)
	# Configure client in the server
	cat << EOF >> /etc/wireguard/wg0.conf
# BEGIN_PEER $client
[Peer]
PublicKey = $(wg pubkey <<< $key)
AllowedIPs = 192.168.10.$octet/32
# END_PEER $client
EOF
	# Create client configuration
	cat << EOF > /etc/wireguard/clients/"$client".conf
[Interface]
PrivateKey = $key
ListenPort = 51820
Address = 192.168.10.$octet/32
DNS = 192.168.1.4

[Peer]
PublicKey = $(grep PrivateKey /etc/wireguard/wg0.conf | cut -d " " -f 3 | wg pubkey)
AllowedIPs = 1.2.3.4/32, 192.168.10.0/24, 192.168.1.0/24
Endpoint = 1.2.3.4:51820
EOF
}

echo "Select an option:"
echo "   1) Add a new client"
echo "   2) List all clients"
echo "   3) Remove an existing client"
echo "   4) Exit"
read -p "Option: " option
until [[ "$option" =~ ^[1-4]$ ]]; do
	echo "$option: invalid selection."
	read -p "Option: " option
done
case "$option" in
	1)
		echo
		echo "Enter a name for the client:"
		read -p "Name: " unsanitized_client
		# Allow a limited set of characters to avoid conflicts
		client=$(sed 's/[^0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ._-]/_/g' <<< "$unsanitized_client")
		while [[ -z "$client" ]] || grep -q "^# BEGIN_PEER $client$" /etc/wireguard/wg0.conf; do
			echo "$client: invalid name."
			read -p "Name: " unsanitized_client
			client=$(sed 's/[^0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ._-]/_/g' <<< "$unsanitized_client")
		done
		echo
		new_client_setup
		# Append new client configuration to the WireGuard interface
		wg addconf wg0 <(sed -n "/^# BEGIN_PEER $client/,/^# END_PEER $client/p" /etc/wireguard/wg0.conf)
		# qrencode -t UTF8 < /etc/wireguard/clients/"$client".conf
		# echo -e '\xE2\x86\x91 That is a QR code containing your client configuration.'
		echo "$client added. Configuration available in: /etc/wireguard/clients/$client.conf"
		echo
                read -p "Print this configuration in stdout? [Y/n]: " stdout_print
                until [[ "$stdout_print" =~ ^[yYnN]*$ ]]; do
                        echo "$stdout_print: invalid selection."
                        read -p "Print this configuration in stdout? [Y/n]: " stdout_print
                done
		if [[ -z "$stdout_print" ]]; then
			stdout_print=Y
		fi
        if [[ "$stdout_print" =~ ^[yY]$ ]]; then
            echo
			cat /etc/wireguard/clients/$client.conf
			echo
        fi
		exit
	;;
	2)
		for client in $(grep '^# BEGIN_PEER' /etc/wireguard/wg0.conf | cut -d ' ' -f 3)
		do
			echo $client "IP:" $(cat /etc/wireguard/wg0.conf | grep -A3 "^# BEGIN_PEER $client" | grep -v '\[Peer\]' | grep -v "PublicKey" | sed -n '1!p' |  cut -d " " -f 3)
			echo
		done
	;;
	3)
		# This option could be documented a bit better and maybe even be simplified
		# ...but what can I say, I want some sleep too
		number_of_clients=$(grep -c '^# BEGIN_PEER' /etc/wireguard/wg0.conf)
		if [[ "$number_of_clients" = 0 ]]; then
			echo
			echo "There are no existing clients!"
			exit
		fi
		echo
		echo "Select the client to remove:"
		grep '^# BEGIN_PEER' /etc/wireguard/wg0.conf | cut -d ' ' -f 3 | nl -s ') '
		read -p "Client: " client_number
		until [[ "$client_number" =~ ^[0-9]+$ && "$client_number" -le "$number_of_clients" ]]; do
			echo "$client_number: invalid selection."
			read -p "Client: " client_number
		done
		client=$(grep '^# BEGIN_PEER' /etc/wireguard/wg0.conf | cut -d ' ' -f 3 | sed -n "$client_number"p)
		echo
		read -p "Confirm $client removal? [y/N]: " remove
		until [[ "$remove" =~ ^[yYnN]*$ ]]; do
			echo "$remove: invalid selection."
			read -p "Confirm $client removal? [y/N]: " remove
		done
		if [[ "$remove" =~ ^[yY]$ ]]; then
			# The following is the right way to avoid disrupting other active connections:
			# Remove from the live interface
			wg set wg0 peer "$(sed -n "/^# BEGIN_PEER $client$/,\$p" /etc/wireguard/wg0.conf | grep -m 1 PublicKey | cut -d " " -f 3)" remove
			# Remove from the configuration file
			sed -i "/^# BEGIN_PEER $client$/,/^# END_PEER $client$/d" /etc/wireguard/wg0.conf
			# Remove config from clients directory
			rm -f /etc/wireguard/clients/$client.conf
			echo
			echo "$client removed!"
		else
			echo
			echo "$client removal aborted!"
		fi
		exit
	;;
	4)
		exit
	;;
esac
